﻿using CtrEditor.Convertidores;
using CtrEditor.Siemens;
using CtrEditor.Simulacion;
using System.Windows;
using System.Windows.Controls;
using nkast.Aether.Physics2D.Common;
using System.Windows.Media.Animation;
using CommunityToolkit.Mvvm.ComponentModel;
namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucDescarte.xaml
    /// </summary>
    public partial class osDescarte : osBase, IosBase
    {
        // Otros datos y métodos relevantes para la simulación
        private simDescarte SimGeometria;

        public static string NombreClase()
        {
            return "Descarte";
        }
        private string nombre = NombreClase();
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        private float diametro;

        partial void OnDiametroChanged(float value)
        {
            SimGeometria?.SetDiameter(Diametro);
        }

        public Vector2 GetCentro()
        {
            return new Vector2(Left + Diametro / 2, Top + Diametro / 2);
        }

        public void SetCentro(float x, float y)
        { Left = x; Top = y; }

        public void SetCentro(Vector2 centro)
        {
            Left = centro.X - Diametro / 2;
            Top = centro.Y - Diametro / 2;
        }

        public override void LeftChanged(float value)
        {
            ActualizarGeometrias();
        }

        public override void TopChanged(float value)
        {
            ActualizarGeometrias();
        }


        private void ActualizarGeometrias()
        {
            if (SimGeometria != null)
            {
                SimGeometria.SetDiameter(Diametro);
                SimGeometria.SetPosition(GetCentro());
            }
        }

        public osDescarte()
        {
            Diametro = 1f;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }
        public override void UpdateGeometryStep()
        {
        }
        public override void UpdatePLC(PLCModel plc, int elapsedMilliseconds)
        {
        }

        public override void UpdateControl(int elapsedMilliseconds)
        {

        }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            ActualizarLeftTop();
            SimGeometria = simulationManager.AddDescarte(Diametro, GetCentro());
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            simulationManager.Remove(SimGeometria);
        }

    }

    public partial class ucDescarte : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucDescarte()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
            Storyboard storyboard = this.Resources["PulsingStoryboard"] as Storyboard;
            if (storyboard != null)
            {
                storyboard.Begin();
            }
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Resize(float width, float height)
        {
            if (Datos is osDescarte datos)
            {
                datos.Diametro = PixelToMeter.Instance.calc.PixelsToMeters(width);
            }
        }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle)
        {
        }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 10;
        }
    }

}
